#!/bin/bash
[ "$#" -lt 1 ] && { echo "At least 1 argument required (the VM name), $# provided."; exit 1; }
[ "$(whoami)" != 'root' ] && { echo "You must be root to run this script!"; exit 1; }

#don't assume we have the helper scripts in the path...
toolpath="/usr/lib/xen_drbd"
server=`cat /etc/xen_drbd/server`
vg=`cat /etc/xen_drbd/vg`

mnum=`cat /etc/xen_drbd/minor`
rmnum=`sudo ssh $server 'cat /etc/xen_drbd/minor'`
[ $rmnum != $mnum ] && { echo "Error! Remote minor number, $rmnum, does not equal the local minor number, $mnum!"; exit 1; }
[ $mnum -gt 200 ] && { echo "Error! Minor number, $mnum, is too high, it may start interfering with other ports! (Like the Xen migration port.)"; exit 1; }



#use alternate vg if supplied
if [ "$#" -ge 2 ]
then
	vg="$2"	
fi

#sanity checks
[ ! -e /dev/${vg}/${1}-disk ] && { echo "The Logical Volume /dev/${vg}/${1}-disk does not exist!"; exit 1; }
[ -e /etc/drbd/${1}.res ] && { echo "Error!  It appears this VM has already been setup for DRBD."; exit 1; }
[ ! -e /etc/xen/${1}.cfg ] && { echo "Error! There is no config script for this VM in /etc/xen.  Is this LV a VM?"; exit 1; }

xm list | egrep "^${1}\s+" &> /dev/null

running=$?

if [ $running -ne 1 ]; then
   echo "Error: VM is running!"
   exit 1;
fi

#let's automatically check to see if we need to replicate the swap.
#you could just unset swap to make it false, but then it would be harder to pass over ssh...
if [ -e /dev/${vg}/${1}-swap ]
then
	echo "Alert: Swap detected."
	swap=1
else
	echo "Alert: Swap not detected."
	swap=0
fi

#extend the disks for the internal metadata
$toolpath/lvdrbdextend.sh /dev/${vg}/${1}-disk

if [ $swap -eq 1 ]
then
	$toolpath/lvdrbdextend.sh /dev/${vg}/${1}-swap

	#drbd won't create metadata on swap volume :/
	dd if=/dev/zero of=/dev/mapper/${vg}-${1}--swap bs=1M count=1
fi

#create the drbd resource file
#this has to be run as sudo, so we want to make sure we have the full path

pnum=`expr $mnum + 7789`
sed "s/resource_name/${1}-disk/g" $toolpath/drbd.res |sed "s/mnum/${mnum}/g" |sed "s/pnum/${pnum}/g" > /etc/drbd.d/${1}.res

if [ $swap -eq 1 ]
then
	mnum=`expr $mnum + 1`
	pnum=`expr $mnum + 7789`
	sed "s/resource_name/${1}-swap/g" $toolpath/drbd.res |sed "s/mnum/${mnum}/g" |sed "s/pnum/${pnum}/g" >> /etc/drbd.d/${1}.res
fi
#increment the mnum
expr $mnum + 1 > /etc/xen_drbd/minor

#create the metadata on the resource
#it always asks... since we know we just extended it, let's just force.
drbdadm -- --force create-md ${1}-disk

#bring them up
drbdadm up ${1}-disk

sdisk=`blockdev --getsz /dev/mapper/${vg}-${1}--disk`
mdisk=`expr $sdisk / 2048`

#do same for swap if we need to.
if [ $swap -eq 1 ]
then
	drbdadm create-md ${1}-swap
	sswap=`blockdev --getsz /dev/mapper/${vg}-${1}--swap`
	mswap=`expr $sswap / 2048`
	drbdadm up ${1}-swap
else
	mswap=0
fi

#ssh key authentication needs to be setup.  note that we are already root (since this script has to be run as root)
ssh $server "$toolpath/vmdrbd_setup_secondary.sh $1 $mnum $swap $mdisk $mswap"

[ $? -ne 0 ] && { echo "Configuring the remote server failed!  Stopping here to ensure remote config isn't overwritten!"; exit 1; }

#temporary fix for migration
#sed "s/RECG-Xen/${vg}/g" /etc/xen/${1}.cfg | sed "s/caixo/${vg}/g" | sed "s/vif.*/vif = [ 'mac= 00:25:90:98:7e:${num}' ]/g" >  /etc/xen/${1}.cfg.tmp && mv  /etc/xen/${1}.cfg.tmp  /etc/xen/${1}.cfg


#convert the cfg file to use drbd
#we have to use phy: rather than drbd: 
if grep -e "hvmloader" -e "pygrub" /etc/xen/${1}.cfg &> /dev/null
then
	$toolpath/convert_hvmcfg.sh $1 $vg
else
	$toolpath/convert_cfg.sh $1 $vg
fi
		
#copy the .cfg file over
scp /etc/xen/${1}.cfg ${server}:/etc/xen/

#set them as primary
#this is actually critical, even though the xen block-drbd script will set them as primary, we need to ensure that they will be the ones replicating the data... 
#otherwise the new lvm on the remote server will populate the data, which would be bad.
drbdadm primary --force ${1}-disk

if [ $swap -eq 1 ]
then
	drbdadm primary --force ${1}-swap

	#we need to set the replicated volume as swap
	mkswap /dev/drbd_${1}-swap
fi

